@@ -5,17 +5,22 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.report import Report
 from trytond.backend import TableHandler, FIELDS
+from trytond.pyson import Equal, Eval, Get, PYSONEncoder
 from decimal import Decimal
 import datetime
-import md5
-import mx.DateTime
+import time
+try:
+    import hashlib
+except ImportError:
+    hashlib = None
+    import md5
 
 _MOVE_STATES = {
-    'readonly': "state == 'posted'",
+    'readonly': Equal(Eval('state'), 'posted'),
 }
 _MOVE_DEPENDS = ['state']
 _LINE_STATES = {
-    'readonly': "state == 'valid'",
+    'readonly': Equal(Eval('state'), 'valid'),
 }
 _LINE_DEPENDS = ['state']
 
@@ -29,7 +34,7 @@ class Move(ModelSQL, ModelView):
     reference = fields.Char('Reference', size=None, readonly=True,
             help='Also known as Folio Number')
     period = fields.Many2One('account.period', 'Period', required=True,
-            states=_MOVE_STATES, depends=_MOVE_DEPENDS)
+            states=_MOVE_STATES, depends=_MOVE_DEPENDS, select=1)
     journal = fields.Many2One('account.journal', 'Journal', required=True,
             states=_MOVE_STATES, depends=_MOVE_DEPENDS)
     date = fields.Date('Effective Date', required=True, states=_MOVE_STATES,
@@ -38,11 +43,14 @@ class Move(ModelSQL, ModelView):
     state = fields.Selection([
         ('draft', 'Draft'),
         ('posted', 'Posted'),
-        ], 'State', required=True, readonly=True)
+        ], 'State', required=True, readonly=True, select=1)
     lines = fields.One2Many('account.move.line', 'move', 'Lines',
             states=_MOVE_STATES, depends=_MOVE_DEPENDS,
-            context="{'journal': journal, 'period': period, " \
-                    "'date': date}")
+            context={
+                'journal': Eval('journal'),
+                'period': Eval('period'),
+                'date': Eval('date'),
+            })
 
     def __init__(self):
         super(Move, self).__init__()
@@ -78,6 +86,9 @@ class Move(ModelSQL, ModelView):
         table = TableHandler(cursor, self, module_name)
         table.index_action(['journal', 'period'], 'add')
 
+        # Add index on create_date
+        table.index_action('create_date', action='add')
+
     def default_period(self, cursor, user, context=None):
         period_obj = self.pool.get('account.period')
         if context is None:
@@ -98,8 +109,7 @@ class Move(ModelSQL, ModelView):
             return period.start_date
         return date_obj.today(cursor, user, context=context)
 
-    def on_change_with_date(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_date(self, cursor, user, vals, context=None):
         line_obj = self.pool.get('account.move.line')
         period_obj = self.pool.get('account.period')
         res = vals['date']
@@ -146,17 +156,12 @@ class Move(ModelSQL, ModelView):
                 return False
         return True
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, ['OR',
-                ('reference', args[i][1], args[i][2]),
-                (self._rec_name, args[i][1], args[i][2]),
-                ], context=context)
-            args2.append(('id', 'in', ids))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, ['OR',
+            ('reference',) + clause[1:],
+            (self._rec_name,) + clause[1:],
+            ], context=context)
+        return [('id', 'in', ids)]
 
     def write(self, cursor, user, ids, vals, context=None):
         res = super(Move, self).write(cursor, user, ids, vals, context=context)
@@ -430,11 +435,11 @@ class Line(ModelSQL, ModelView):
     _description = __doc__
 
     name = fields.Char('Name', size=None, required=True)
-    debit = fields.Numeric('Debit', digits="(16, currency_digits)",
+    debit = fields.Numeric('Debit', digits=(16, Eval('currency_digits', 2)),
             on_change=['account', 'debit', 'credit', 'tax_lines',
                 'journal', 'move'], depends=['currency_digits', 'credit',
                     'tax_lines', 'journal'])
-    credit = fields.Numeric('Credit', digits="(16, currency_digits)",
+    credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
             on_change=['account', 'debit', 'credit', 'tax_lines',
                 'journal', 'move'], depends=['currency_digits', 'debit',
                     'tax_lines', 'journal'])
@@ -443,20 +448,23 @@ class Line(ModelSQL, ModelView):
             select=1,
             on_change=['account', 'debit', 'credit', 'tax_lines',
                 'journal', 'move'])
-    move = fields.Many2One('account.move', 'Move', states=_LINE_STATES,
-            select=1, required=True)
-    journal = fields.Function('get_move_field', fnct_inv='set_move_field',
-            type='many2one', relation='account.journal', string='Journal',
-            fnct_search='search_move_field')
-    period = fields.Function('get_move_field', fnct_inv='set_move_field',
-            type='many2one', relation='account.period', string='Period',
-            fnct_search='search_move_field')
-    date = fields.Function('get_move_field', fnct_inv='set_move_field',
-            type='date', string='Effective Date', required=True,
-            fnct_search='search_move_field')
+    move = fields.Many2One('account.move', 'Move', select=1, required=True,
+            states={
+                'required': False,
+                'readonly': Equal(Eval('state'), 'valid'),
+            })
+    journal = fields.Function(fields.Many2One('account.journal', 'Journal'),
+            'get_move_field', setter='set_move_field',
+            searcher='search_move_field')
+    period = fields.Function(fields.Many2One('account.period', 'Period'),
+            'get_move_field', setter='set_move_field',
+            searcher='search_move_field')
+    date = fields.Function(fields.Date('Effective Date', required=True),
+            'get_move_field', setter='set_move_field',
+            searcher='search_move_field')
     reference = fields.Char('Reference', size=None)
     amount_second_currency = fields.Numeric('Amount Second Currency',
-            digits="(16, second_currency_digits)",
+            digits=(16, Eval('second_currency_digits', 2)),
             help='The amount expressed in a second currency',
             depends=['second_currency_digits'])
     second_currency = fields.Many2One('currency.currency', 'Second Currency',
@@ -471,20 +479,19 @@ class Line(ModelSQL, ModelView):
     state = fields.Selection([
         ('draft', 'Draft'),
         ('valid', 'Valid'),
-        ], 'State', readonly=True, required=True)
+        ], 'State', readonly=True, required=True, select=1)
     active = fields.Boolean('Active', select=2)
     reconciliation = fields.Many2One('account.move.reconciliation',
             'Reconciliation', readonly=True, ondelete='SET NULL', select=2)
     tax_lines = fields.One2Many('account.tax.line', 'move_line', 'Tax Lines')
-    move_state = fields.Function('get_move_field', type='selection',
-            selection=[
-                ('draft', 'Draft'),
-                ('posted', 'Posted'),
-            ], string='Move State', fnct_search='search_move_field')
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits')
-    second_currency_digits = fields.Function('get_currency_digits',
-            type='integer', string='Second Currency Digits')
+    move_state = fields.Function(fields.Selection([
+        ('draft', 'Draft'),
+        ('posted', 'Posted'),
+        ], 'Move State'), 'get_move_field', searcher='search_move_field')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+            'get_currency_digits')
+    second_currency_digits = fields.Function(fields.Integer(
+        'Second Currency Digits'), 'get_currency_digits')
 
     def __init__(self):
         super(Line, self).__init__()
@@ -515,6 +522,8 @@ class Line(ModelSQL, ModelView):
         super(Line, self).init(cursor, module_name)
 
         table = TableHandler(cursor, self, module_name)
+        # Index for General Ledger
+        table.index_action(['move', 'account'], 'add')
 
         # Migration from 1.2
         table.not_null_action('blocked', action='remove')
@@ -623,16 +632,19 @@ class Line(ModelSQL, ModelView):
                             code_id = tax.credit_note_tax_code.id
                             account_id = tax.credit_note_account.id
                     if base_id in line_code_taxes or not base_id:
-                        taxes.setdefault((account_id, code_id), False)
+                        taxes.setdefault((account_id, code_id, tax.id), False)
                 for tax_line in line.tax_lines:
-                    taxes[(line.account.id, tax_line.code.id)] = True
-                if not line.tax_lines and line.account.taxes:
-                    if line.account.id in no_code_taxes:
-                        taxes[(line.account.id, False)] = True
-                    else:
-                        no_code_taxes.append(line.account.id)
-                elif not line.tax_lines:
-                    taxes[(line.account.id, False)] = True
+                    taxes[(line.account.id, tax_line.code.id,
+                            tax_line.tax.id)] = True
+                if not line.tax_lines:
+                    no_code_taxes.append(line.account.id)
+        for no_code_account_id in no_code_taxes:
+            for (account_id, code_id, tax_id), test in \
+                    taxes.iteritems():
+                if (not test
+                        and not code_id
+                        and no_code_account_id == account_id):
+                    taxes[(account_id, code_id, tax_id)] = True
 
         if 'account' in fields:
             if total >= Decimal('0.0'):
@@ -647,8 +659,8 @@ class Line(ModelSQL, ModelView):
             values.setdefault('credit', total > 0 and total or False)
 
         if move.journal.type in ('expense', 'revenue'):
-            for account_id, code_id in taxes:
-                if taxes[(account_id, code_id)]:
+            for account_id, code_id, tax_id in taxes:
+                if taxes[(account_id, code_id, tax_id)]:
                     continue
                 for line in move.lines:
                     if move.journal.type == 'revenue':
@@ -668,8 +680,9 @@ class Line(ModelSQL, ModelView):
                             line.debit or line.credit, 1, context=context):
                         if (tax_line['tax'][key + '_account'].id \
                                 or line.account.id) == account_id \
-                            and tax_line['tax'][key + '_tax_code'].id \
-                                    == code_id:
+                                and tax_line['tax'][key + '_tax_code'].id \
+                                    == code_id \
+                                and tax_line['tax'].id == tax_id:
                             if line.debit:
                                 line_amount += tax_line['amount']
                             else:
@@ -698,11 +711,14 @@ class Line(ModelSQL, ModelView):
                                 'code': code_id,
                                 'code.rec_name': tax_code_obj.browse(cursor,
                                     user, code_id, context=context).rec_name,
+                                'tax': tax_id,
+                                'tax.rec_name': tax_obj.browse(cursor, user,
+                                    tax_id, context=context).rec_name
                             },
                         ]
         return values
 
-    def get_currency_digits(self, cursor, user, ids, names, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, names, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             for name in names:
@@ -715,7 +731,7 @@ class Line(ModelSQL, ModelView):
                         res[name][line.id] = line.account.second_currency.digits
         return res
 
-    def on_change_debit(self, cursor, user, ids, vals, context=None):
+    def on_change_debit(self, cursor, user, vals, context=None):
         res = {}
         if context is None:
             context = {}
@@ -726,14 +742,14 @@ class Line(ModelSQL, ModelView):
                     context=context)
             if journal.type in ('expense', 'revenue'):
                 res['tax_lines'] = self._compute_tax_lines(cursor, user,
-                        ids, vals, journal.type, context=context)
+                        vals, journal.type, context=context)
                 if not res['tax_lines']:
                     del res['tax_lines']
         if vals.get('debit'):
             res['credit'] = Decimal('0.0')
         return res
 
-    def on_change_credit(self, cursor, user, ids, vals, context=None):
+    def on_change_credit(self, cursor, user, vals, context=None):
         res = {}
         if context is None:
             context = {}
@@ -744,14 +760,14 @@ class Line(ModelSQL, ModelView):
                     context=context)
             if journal.type in ('expense', 'revenue'):
                 res['tax_lines'] = self._compute_tax_lines(cursor, user,
-                        ids, vals, journal.type, context=context)
+                        vals, journal.type, context=context)
                 if not res['tax_lines']:
                     del res['tax_lines']
         if vals.get('credit'):
             res['debit'] = Decimal('0.0')
         return res
 
-    def on_change_account(self, cursor, user, ids, vals, context=None):
+    def on_change_account(self, cursor, user, vals, context=None):
         account_obj = self.pool.get('account.account')
         journal_obj = self.pool.get('account.journal')
 
@@ -764,7 +780,7 @@ class Line(ModelSQL, ModelView):
                     context['journal'], context=context)
             if journal.type in ('expense', 'revenue'):
                 res['tax_lines'] = self._compute_tax_lines(cursor, user,
-                        ids, vals, journal.type, context=context)
+                        vals, journal.type, context=context)
                 if not res['tax_lines']:
                     del res['tax_lines']
 
@@ -776,7 +792,7 @@ class Line(ModelSQL, ModelView):
                 res['second_currency_digits'] = account.second_currency.digits
         return res
 
-    def _compute_tax_lines(self, cursor, user, ids, vals, journal_type,
+    def _compute_tax_lines(self, cursor, user, vals, journal_type,
             context=None):
         res = {}
         account_obj = self.pool.get('account.account')
@@ -809,22 +825,28 @@ class Line(ModelSQL, ModelView):
                         [x.id for x in account.taxes],
                         debit or credit, 1, context=context):
                     code_id = tax_line['tax'][key + '_base_code'].id
-                    base_amounts.setdefault(code_id, Decimal('0.0'))
-                    base_amounts[code_id] += tax_line['base'] * \
-                            tax_line['tax'][key + '_tax_sign']
-                for code_id in base_amounts:
                     if not code_id:
                         continue
+                    tax_id = tax_line['tax'].id
+                    base_amounts.setdefault((code_id, tax_id), Decimal('0.0'))
+                    base_amounts[code_id, tax_id] += tax_line['base'] * \
+                            tax_line['tax'][key + '_tax_sign']
+                for code_id, tax_id in base_amounts:
+                    if not base_amounts[code_id, tax_id]:
+                        continue
                     res.setdefault('add', []).append({
-                        'amount': base_amounts[code_id],
+                        'amount': base_amounts[code_id, tax_id],
                         'currency_digits': account.currency_digits,
                         'code': code_id,
                         'code.rec_name': tax_code_obj.browse(cursor, user,
-                            code_id, context=context).rec_name,
+                                code_id, context=context).rec_name,
+                        'tax': tax_id,
+                        'tax.rec_name': tax_obj.browse(cursor, user,
+                                tax_id, context=context).rec_name
                     })
         return res
 
-    def on_change_party(self, cursor, user, ids, vals, context=None):
+    def on_change_party(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         journal_obj = self.pool.get('account.journal')
         account_obj = self.pool.get('account.account')
@@ -853,7 +875,7 @@ class Line(ModelSQL, ModelView):
             amount = cursor.fetchone()[0]
             # SQLite uses float for SUM
             if not isinstance(amount, Decimal):
-                amount = Decimal(str(amount))
+                amout = Decimal(str(amount))
             if not currency_obj.is_zero(cursor, user,
                     party.account_receivable.currency, amount):
                 if amount > Decimal('0.0'):
@@ -916,7 +938,7 @@ class Line(ModelSQL, ModelView):
                     res['account.rec_name'] = party.account_payable.rec_name
         return res
 
-    def get_move_field(self, cursor, user, ids, name, arg, context=None):
+    def get_move_field(self, cursor, user, ids, name, context=None):
         if name == 'move_state':
             name = 'state'
         if name not in ('period', 'journal', 'date', 'state'):
@@ -929,7 +951,7 @@ class Line(ModelSQL, ModelView):
                 res[line.id] = line.move[name].id
         return res
 
-    def set_move_field(self, cursor, user, id, name, value, arg, context=None):
+    def set_move_field(self, cursor, user, ids, name, value, context=None):
         if name == 'move_state':
             name = 'state'
         if name not in ('period', 'journal', 'date', 'state'):
@@ -937,21 +959,15 @@ class Line(ModelSQL, ModelView):
         if not value:
             return
         move_obj = self.pool.get('account.move')
-        line = self.browse(cursor, user, id, context=context)
-        move_obj.write(cursor, user, line.move.id, {
+        lines = self.browse(cursor, user, ids, context=context)
+        move_obj.write(cursor, user, [line.move.id for line in lines], {
             name: value,
             }, context=context)
 
-    def search_move_field(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            field = args[i][0]
-            if args[i][0] == 'move_state':
-                field = 'state'
-            args2.append(('move.' + field, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_move_field(self, cursor, user, name, clause, context=None):
+        if name == 'move_state':
+            name = 'state'
+        return [('move.' + name,) + clause[1:]]
 
     def query_get(self, cursor, user, obj='l', context=None):
         '''
@@ -969,7 +985,7 @@ class Line(ModelSQL, ModelView):
             context = {}
 
         if context.get('date'):
-            mx.DateTime.strptime(str(context['date']), '%Y-%m-%d')
+            time.strptime(str(context['date']), '%Y-%m-%d')
             fiscalyear_ids = fiscalyear_obj.search(cursor, user, [
                 ('start_date', '<=', context['date']),
                 ('end_date', '>=', context['date']),
@@ -1258,7 +1274,10 @@ class Line(ModelSQL, ModelView):
             result['fields'] = self.fields_get(cursor, user,
                     fields_names=list(fields), context=context)
             del result['md5']
-            result['md5'] = md5.new(str(result)).hexdigest()
+            if hashlib:
+                result['md5'] = hashlib.md5(str(result)).hexdigest()
+            else:
+                result['md5'] = md5.new(str(result)).hexdigest()
             if hexmd5 == result['md5']:
                 return True
         return result
@@ -1348,8 +1367,11 @@ class OpenJournalAsk(ModelView):
     _description = __doc__
     journal = fields.Many2One('account.journal', 'Journal', required=True)
     period = fields.Many2One('account.period', 'Period', required=True,
-            domain=[('state', '!=', 'close'),
-                "('fiscalyear.company.id', '=', context.get('company', False))"])
+            domain=[
+                ('state', '!=', 'close'),
+                ('fiscalyear.company.id', '=',
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
 
     def default_period(self, cursor, user, context=None):
         period_obj = self.pool.get('account.period')
@@ -1437,21 +1459,16 @@ class OpenJournal(Wizard):
                 'period': period.id,
                 }, context=context)
 
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_move_line_form'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_move_line_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
         # Remove name to use the one from view_header_get
         del res['name']
-        res['domain'] = str([
+        res['pyson_domain'] = PYSONEncoder().encode([
             ('journal', '=', journal_id),
             ('period', '=', period_id),
             ])
-        res['context'] = str({
+        res['pyson_context'] = PYSONEncoder().encode({
             'journal': journal_id,
             'period': period_id,
             })
@@ -1493,22 +1510,19 @@ class OpenAccount(Wizard):
             for period in fiscalyear.periods:
                 period_ids.append(period.id)
 
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_move_line_form'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
-        res['domain'] = [
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_move_line_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_domain'] = [
             ('period', 'in', period_ids),
             ('account', '=', data['id']),
             ]
         if context.get('posted'):
-            res['domain'].append(('move.state', '=', 'posted'))
-        res['domain'] = str(res['domain'])
-        res['context'] = str({'fiscalyear': context.get('fiscalyear')})
+            res['pyson_domain'].append(('move.state', '=', 'posted'))
+        res['pyson_domain'] = PYSONEncoder().encode(res['pyson_domain'])
+        res['pyson_context'] = PYSONEncoder().encode({
+            'fiscalyear': context.get('fiscalyear'),
+        })
         return res
 
 OpenAccount()
@@ -1676,16 +1690,10 @@ class OpenReconcileLines(Wizard):
     def _action_open_reconcile_lines(self, cursor, user, data, context=None):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_move_line_form'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
-        res['domain'] = str([
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_move_line_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_domain'] = PYSONEncoder().encode([
             ('account', '=', data['form']['account']),
             ('reconciliation', '=', False),
             ])
@@ -1736,7 +1744,6 @@ class PrintGeneralJournalInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
