@@ -6,6 +6,8 @@ from trytond.wizard import Wizard
 from decimal import Decimal
 from trytond.tools import Cache
 from trytond.backend import TableHandler
+from trytond.pyson import Eval, Not, Equal, If, In, Bool, Get, Or, And, \
+        PYSONEncoder
 
 
 class Group(ModelSQL, ModelView):
@@ -16,11 +18,12 @@ class Group(ModelSQL, ModelView):
     name = fields.Char('Name', size=None, required=True, translate=True)
     code = fields.Char('Code', size=None, required=True)
 
-    def __init__(self):
-        super(Group, self).__init__()
-        self._sql_constraints += [
-            ('code_uniq', 'UNIQUE(code)', 'The code must be unique!'),
-        ]
+    def init(self, cursor, module_name):
+        super(Group, self).init(cursor, module_name)
+        table = TableHandler(cursor, self, module_name)
+
+        # Migration from 1.4 drop code_uniq constraint
+        table.drop_constraint('code_uniq')
 
 Group()
 
@@ -36,6 +39,7 @@ class CodeTemplate(ModelSQL, ModelView):
     childs = fields.One2Many('account.tax.code.template', 'parent', 'Children')
     account = fields.Many2One('account.account.template', 'Account Template',
             domain=[('parent', '=', False)], required=True)
+    description = fields.Text('Description', translate=True)
 
     def __init__(self):
         super(CodeTemplate, self).__init__()
@@ -65,6 +69,8 @@ class CodeTemplate(ModelSQL, ModelView):
             res['name'] = template.name
         if not code or code.code != template.code:
             res['code'] = template.code
+        if not code or code.description != template.description:
+            res['description'] = template.description
         if not code or code.template.id != template.id:
             res['template'] = template.id
         return res
@@ -106,6 +112,10 @@ class CodeTemplate(ModelSQL, ModelView):
             new_id = tax_code_obj.create(cursor, user, vals, context=context)
 
             prev_lang = template._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -114,9 +124,10 @@ class CodeTemplate(ModelSQL, ModelView):
                 ctx['language'] = lang
                 template.setLang(lang)
                 data = {}
-                for field in template._columns.keys():
-                    if template._columns[field].translate:
-                        data[field] = template[field]
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     tax_code_obj.write(cursor, user, new_id, data, context=ctx)
             template.setLang(prev_lang)
@@ -145,14 +156,16 @@ class Code(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=2)
     company = fields.Many2One('company.company', 'Company', required=True)
     parent = fields.Many2One('account.tax.code', 'Parent', select=1,
-            domain="[('company', '=', company)]", depends=['company'])
+            domain=[('company', '=', Eval('company', 0))], depends=['company'])
     childs = fields.One2Many('account.tax.code', 'parent', 'Children',
-            domain="[('company', '=', company)]", depends=['company'])
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits', on_change_with=['company'])
-    sum = fields.Function('get_sum', digits="(16, currency_digits)",
-            string='Sum', depends=['currency_digits'])
+            domain=[('company', '=', Eval('company', 0))], depends=['company'])
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['company']), 'get_currency_digits')
+    sum = fields.Function(fields.Numeric('Sum',
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+        'get_sum')
     template = fields.Many2One('account.tax.code.template', 'Template')
+    description = fields.Text('Description', translate=True)
 
     def __init__(self):
         super(Code, self).__init__()
@@ -168,15 +181,13 @@ class Code(ModelSQL, ModelView):
         return True
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
             return context['company']
         return False
 
-    def on_change_with_currency_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
         company_obj = self.pool.get('company.company')
         if vals.get('company'):
             company = company_obj.browse(cursor, user, vals['company'],
@@ -184,13 +195,13 @@ class Code(ModelSQL, ModelView):
             return company.currency.digits
         return 2
 
-    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for code in self.browse(cursor, user, ids, context=context):
             res[code.id] = code.company.currency.digits
         return res
 
-    def get_sum(self, cursor, user, ids, name, arg, context=None):
+    def get_sum(self, cursor, user, ids, name, context=None):
         res = {}
         move_line_obj = self.pool.get('account.move.line')
         currency_obj = self.pool.get('currency.currency')
@@ -207,12 +218,15 @@ class Code(ModelSQL, ModelView):
                 'WHERE c.id = tl.code ' \
                     'AND tl.move_line = l.id ' \
                     'AND c.id IN (' + \
-                        ','.join(['%s' for x in all_ids])+ ') ' \
+                        ','.join(('%s',) * len(all_ids))+ ') ' \
                     'AND ' + line_query + ' ' \
                     'AND c.active ' \
                 'GROUP BY c.id', all_ids)
         code_sum = {}
         for code_id, sum in cursor.fetchall():
+            # SQLite uses float for SUM
+            if not isinstance(sum, Decimal):
+                sum = Decimal(str(sum))
             code_sum[code_id] = sum
 
         for code in self.browse(cursor, user, ids, context=context):
@@ -228,7 +242,7 @@ class Code(ModelSQL, ModelView):
                     code.company.currency, res[code.id])
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -239,18 +253,12 @@ class Code(ModelSQL, ModelView):
                 res[code.id] = code.name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, [('code', args[i][1], args[i][2])],
-                    context=context)
-            if ids:
-                args2.append(('code', args[i][1], args[i][2]))
-            else:
-                args2.append(('name', args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, [('code',) + clause[1:]], limit=1,
+                order=[], context=context)
+        if ids:
+            return [('code',) + clause[1:]]
+        return [('name',) + clause[1:]]
 
     def delete(self, cursor, user, ids, context=None):
         if isinstance(ids, (int, long)):
@@ -290,6 +298,10 @@ class Code(ModelSQL, ModelView):
                 self.write(cursor, user, code.id, vals, context=context)
 
             prev_lang = code._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in code.template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = code.template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -297,8 +309,12 @@ class Code(ModelSQL, ModelView):
                     continue
                 ctx['language'] = lang
                 code.setLang(lang)
-                data = template_obj._get_tax_code_value(cursor, user,
-                                code.template, context=ctx, code=code)
+                data = {}
+                for field_name, field in code.template._columns.iteritems():
+                    if (getattr(field, 'translate', False)
+                            and code.template[field_name] !=
+                            prev_data[field_name]):
+                        data[field_name] = code.template[field_name]
                 if data:
                     self.write(cursor, user, code.id, data, context=ctx)
             code.setLang(prev_lang)
@@ -322,12 +338,12 @@ class OpenChartCodeInit(ModelView):
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
             help='Leave empty for all open fiscal year',
             states={
-                'invisible': "method != 'fiscalyear'",
+                'invisible': Not(Equal(Eval('method'), 'fiscalyear')),
             }, depends=['method'])
     periods = fields.Many2Many('account.period', None, None, 'Periods',
             help='Leave empty for all periods of all open fiscal year',
             states={
-                'invisible': "method != 'periods'",
+                'invisible': Not(Equal(Eval('method'), 'periods')),
             }, depends=['method'])
 
     def default_method(self, cursor, user, context=None):
@@ -362,20 +378,17 @@ class OpenChartCode(Wizard):
     def _action_open_chart(self, cursor, user, data, context=None):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_tax_code_tree2'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id,
-                context=context)
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_tax_code_tree2', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
         if data['form']['method'] == 'fiscalyear':
-            res['context'] = str({'fiscalyear': data['form']['fiscalyear']})
+            res['pyson_context'] = PYSONEncoder().encode({
+                'fiscalyear': data['form']['fiscalyear'],
+            })
         else:
-            res['context'] = str({'periods': data['form']['periods'][0][1]})
+            res['pyson_context'] = PYSONEncoder().encode({
+                'periods': data['form']['periods'][0][1],
+            })
         return res
 
 OpenChartCode()
@@ -390,7 +403,7 @@ class TaxTemplate(ModelSQL, ModelView):
     description = fields.Char('Description', required=True, translate=True)
     group = fields.Many2One('account.tax.group', 'Group')
     sequence = fields.Integer('Sequence')
-    amount = fields.Numeric('Amount', digits=(16, 2))
+    amount = fields.Numeric('Amount', digits=(16, 8))
     percentage = fields.Numeric('Percentage', digits=(16, 8))
     type = fields.Selection([
         ('percentage', 'Percentage'),
@@ -545,6 +558,10 @@ class TaxTemplate(ModelSQL, ModelView):
             new_id = tax_obj.create(cursor, user, vals, context=context)
 
             prev_lang = template._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -553,9 +570,10 @@ class TaxTemplate(ModelSQL, ModelView):
                 ctx['language'] = lang
                 template.setLang(lang)
                 data = {}
-                for field in template._columns.keys():
-                    if template._columns[field].translate:
-                        data[field] = template[field]
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     tax_obj.write(cursor, user, new_id, data, context=ctx)
             template.setLang(prev_lang)
@@ -590,18 +608,20 @@ class Tax(ModelSQL, ModelView):
             help="The name that will be used in reports")
     group = fields.Many2One('account.tax.group', 'Group',
             states={
-                'invisible': "locals().get('parent', True)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     active = fields.Boolean('Active')
     sequence = fields.Integer('Sequence',
             help='Use to order the taxes')
-    amount = fields.Numeric('Amount', digits=(16, 2),
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['company']), 'get_currency_digits')
+    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
             states={
-                'invisible': "type != 'fixed'",
-            }, help='In company\'s currency', depends=['type'])
+                'invisible': Not(Equal(Eval('type'), 'fixed')),
+            }, help='In company\'s currency', depends=['type', 'currency_digits'])
     percentage = fields.Numeric('Percentage', digits=(16, 8),
             states={
-                'invisible': "type != 'percentage'",
+                'invisible': Not(Equal(Eval('type'), 'percentage')),
             }, help='In %', depends=['type'])
     type = fields.Selection([
         ('percentage', 'Percentage'),
@@ -611,60 +631,72 @@ class Tax(ModelSQL, ModelView):
     parent = fields.Many2One('account.tax', 'Parent', ondelete='CASCADE')
     childs = fields.One2Many('account.tax', 'parent', 'Children')
     company = fields.Many2One('company.company', 'Company', required=True,
-            domain="[('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))]")
+            domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     invoice_account = fields.Many2One('account.account', 'Invoice Account',
-            domain="[('company', '=', company)]",
+            domain=[
+                ('company', '=', Eval('company')),
+                ('kind', '!=', 'view'),
+            ],
             states={
-                'readonly': "type == 'none' or not company",
-                'required': "type != 'none' and company",
+                'readonly': Or(Equal(Eval('type'), 'none'),
+                    Not(Bool(Eval('company')))),
+                'required': And(Not(Equal(Eval('type'), 'none')),
+                    Bool(Eval('company'))),
             }, depends=['company'])
     credit_note_account = fields.Many2One('account.account', 'Credit Note Account',
-            domain="[('company', '=', company)]",
+            domain=[
+                ('company', '=', Eval('company')),
+                ('kind', '!=', 'view'),
+            ],
             states={
-                'readonly': "type == 'none' or not company",
-                'required': "type != 'none' and company",
+                'readonly': Or(Equal(Eval('type'), 'none'),
+                    Not(Bool(Eval('company')))),
+                'required': And(Not(Equal(Eval('type'), 'none')),
+                    Bool(Eval('company'))),
             }, depends=['company', 'type'])
 
     invoice_base_code = fields.Many2One('account.tax.code',
             'Invoice Base Code',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     invoice_base_sign = fields.Numeric('Invoice Base Sign', digits=(2, 0),
             help='Usualy 1 or -1',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     invoice_tax_code = fields.Many2One('account.tax.code',
             'Invoice Tax Code',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     invoice_tax_sign = fields.Numeric('Invoice Tax Sign', digits=(2, 0),
             help='Usualy 1 or -1',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     credit_note_base_code = fields.Many2One('account.tax.code',
             'Credit Note Base Code',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     credit_note_base_sign = fields.Numeric('Credit Note Base Sign', digits=(2, 0),
             help='Usualy 1 or -1',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     credit_note_tax_code = fields.Many2One('account.tax.code',
             'Credit Note Tax Code',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     credit_note_tax_sign = fields.Numeric('Credit Note Tax Sign', digits=(2, 0),
             help='Usualy 1 or -1',
             states={
-                'readonly': "type == 'none'",
+                'readonly': Equal(Eval('type'), 'none'),
             }, depends=['type'])
     template = fields.Many2One('account.tax.template', 'Template')
 
@@ -701,13 +733,26 @@ class Tax(ModelSQL, ModelView):
         return Decimal('1')
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
             return context['company']
         return False
 
+    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
+        company_obj = self.pool.get('company.company')
+        if vals.get('company'):
+            company = company_obj.browse(cursor, user, vals['company'],
+                    context=context)
+            return company.currency.digits
+        return 2
+
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
+        res = {}
+        for tax in self.browse(cursor, user, ids, context=context):
+            res[tax.id] = tax.company.currency.digits
+        return res
+
     def _process_tax(self, cursor, user, tax, price_unit, context=None):
         if tax.type == 'percentage':
             amount = price_unit * tax.percentage / Decimal('100')
@@ -750,6 +795,7 @@ class Tax(ModelSQL, ModelView):
         self.sort_taxes(cursor.dbname)
         return super(Tax, self).write(cursor, user, ids, vals, context=context)
 
+    @Cache('account_tax.sort_taxes')
     def sort_taxes(self, cursor, user, ids, context=None):
         '''
         Return a list of taxe ids sorted
@@ -764,8 +810,6 @@ class Tax(ModelSQL, ModelView):
             ('id', 'in', ids),
             ], order=[('sequence', 'ASC'), ('id', 'ASC')], context=context)
 
-    sort_taxes = Cache('account_tax.sort_taxes')(sort_taxes)
-
     def compute(self, cursor, user, ids, price_unit, quantity, context=None):
         '''
         Compute taxes for price_unit and quantity.
@@ -880,6 +924,10 @@ class Tax(ModelSQL, ModelView):
                 self.write(cursor, user, tax.id, vals, context=context)
 
             prev_lang = tax._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in tax.template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = tax.template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -887,8 +935,12 @@ class Tax(ModelSQL, ModelView):
                     continue
                 ctx['language'] = lang
                 tax.setLang(lang)
-                data = template_obj._get_tax_value(cursor, user, tax.template,
-                        context=ctx, tax=tax)
+                data = {}
+                for field_name, field in tax.template._columns.iteritems():
+                    if (getattr(field, 'translate', False)
+                            and tax.template[field_name] !=
+                            prev_data[field_name]):
+                        data[field_name] = tax.template[field_name]
                 if data:
                     self.write(cursor, user, tax.id, data, context=ctx)
             tax.setLang(prev_lang)
@@ -907,11 +959,36 @@ class Line(ModelSQL, ModelView):
     _description = __doc__
     _rec_name = 'amount'
 
-    amount = fields.Numeric('Amount', digits=(16, 2))
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['move_line']), 'get_currency_digits')
+    amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
+            depends=['currency_digits'])
     code = fields.Many2One('account.tax.code', 'Code', select=1, required=True)
+    tax = fields.Many2One('account.tax', 'Tax', select=1, ondelete='RESTRICT',
+            on_change=['tax'])
     move_line = fields.Many2One('account.move.line', 'Move Line',
             required=True, select=1, ondelete='CASCADE')
 
+    def on_change_with_currency_digits(self, cursor, user, vals, context=None):
+        move_line_obj = self.pool.get('account.move.line')
+        if vals.get('move_line'):
+            move_line = move_line_obj.browse(cursor, user, vals['move_line'],
+                    context=context)
+            return move_line.currency_digits
+        return 2
+
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
+        res = {}
+        for line in self.browse(cursor, user, ids, context=context):
+            res[line.id] = line.move_line.currency_digits
+        return res
+
+    def on_change_tax(self, cursor, user, vals, context=None):
+        res = {
+            'code': False,
+            }
+        return res
+
 Line()
 
 
@@ -976,6 +1053,10 @@ class RuleTemplate(ModelSQL, ModelView):
             new_id = rule_obj.create(cursor, user, vals, context=context)
 
             prev_lang = template._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -984,9 +1065,10 @@ class RuleTemplate(ModelSQL, ModelView):
                 ctx['language'] = lang
                 template.setLang(lang)
                 data = {}
-                for field in template._columns.keys():
-                    if template._columns[field].translate:
-                        data[field] = template[field]
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     rule_obj.write(cursor, user, new_id, data, context=ctx)
             template.setLang(prev_lang)
@@ -1004,8 +1086,10 @@ class Rule(ModelSQL, ModelView):
     _description = __doc__
     name = fields.Char('Name', required=True, translate=True)
     company = fields.Many2One('company.company', 'Company', required=True,
-            select=1, domain="[('id', 'company' in context '=' or '!=', " \
-                    "context.get('company', 0))]")
+            select=1, domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     lines = fields.One2Many('account.tax.rule.line', 'rule', 'Lines')
     template = fields.Many2One('account.tax.rule.template', 'Template')
 
@@ -1025,14 +1109,15 @@ class Rule(ModelSQL, ModelView):
         tax_obj = self.pool.get('account.tax')
         rule_line_obj = self.pool.get('account.tax.rule.line')
 
-        if isinstance(rule, (int, long)):
+        if isinstance(rule, (int, long)) and rule:
             rule = self.browse(cursor, user, rule, context=context)
 
-        if isinstance(tax, (int, long)):
+        if isinstance(tax, (int, long)) and tax:
             tax = tax_obj.browse(cursor, user, tax, context=context)
 
         pattern = pattern.copy()
         pattern['group'] = tax and tax.group.id or False
+        pattern['origin_tax'] = tax and tax.id or False
 
         for line in rule.lines:
             if rule_line_obj.match(cursor, user, line, pattern,
@@ -1069,6 +1154,10 @@ class Rule(ModelSQL, ModelView):
                 self.write(cursor, user, rule.id, vals, context=context)
 
             prev_lang = rule._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in rule.template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = rule.template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -1076,8 +1165,12 @@ class Rule(ModelSQL, ModelView):
                     continue
                 ctx['language'] = lang
                 rule.setLang(lang)
-                data = template_obj._get_tax_rule_value(cursor, user,
-                        rule.template, context=ctx, rule=rule)
+                data = {}
+                for field_name, field in rule.template._columns.iteritems():
+                    if (getattr(field, 'translate', False)
+                            and rule.template[field_name] !=
+                            prev_data[field_name]):
+                        data[field_name] = rule.template[field_name]
                 if data:
                     self.write(cursor, user, rule.id, data, context=ctx)
             rule.setLang(prev_lang)
@@ -1093,9 +1186,18 @@ class RuleLineTemplate(ModelSQL, ModelView):
     rule = fields.Many2One('account.tax.rule.template', 'Rule', required=True,
             ondelete='CASCADE')
     group = fields.Many2One('account.tax.group', 'Tax Group')
+    origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
+            domain=[
+                ('account', '=', Get(Eval('_parent_rule', {}), 'account')),
+                ('group', '=', Eval('group'))
+            ],
+            help='If the original tax template is filled, the rule will be ' \
+                    'applied only for this tax template.')
     tax = fields.Many2One('account.tax.template', 'Substitution Tax',
-            domain="[('account', '=', _parent_rule.account), " \
-                    "('group', '=', group)]")
+            domain=[
+                ('account', '=', Get(Eval('_parent_rule', {}), 'account')),
+                ('group', '=', Eval('group')),
+            ])
     sequence = fields.Integer('Sequence')
 
     def __init__(self):
@@ -1118,6 +1220,8 @@ class RuleLineTemplate(ModelSQL, ModelView):
         res = {}
         if not rule_line or rule_line.group.id != template.group.id:
             res['group'] = template.group.id
+        if not rule_line or rule_line.origin_tax.id != template.origin_tax.id:
+            res['origin_tax'] = template.origin_tax.id
         if not rule_line or rule_line.sequence != template.sequence:
             res['sequence'] = template.sequence
         if not rule_line or rule_line.template.id != template.id:
@@ -1158,6 +1262,10 @@ class RuleLineTemplate(ModelSQL, ModelView):
             vals = self._get_tax_rule_line_value(cursor, user, template,
                     context=context)
             vals['rule'] = template2rule[template.rule.id]
+            if template.origin_tax:
+                vals['origin_tax'] = template2tax[template.origin_tax.id]
+            else:
+                vals['origin_tax'] = False
             if template.tax:
                 vals['tax'] = template2tax[template.tax.id]
             else:
@@ -1179,9 +1287,18 @@ class RuleLine(ModelSQL, ModelView):
     rule = fields.Many2One('account.tax.rule', 'Rule', required=True,
             select=1, ondelete='CASCADE')
     group = fields.Many2One('account.tax.group', 'Tax Group')
+    origin_tax = fields.Many2One('account.tax', 'Original Tax',
+            domain=[
+                ('company', '=', Get(Eval('_parent_rule', {}), 'company')),
+                ('group', '=', Eval('group')),
+            ],
+            help='If the original tax is filled, the rule will be applied ' \
+                    'only for this tax.')
     tax = fields.Many2One('account.tax', 'Substitution Tax',
-            domain="[('company', '=', _parent_rule.company), " \
-                    "('group', '=', group)]")
+            domain=[
+                ('company', '=', Get(Eval('_parent_rule', {}), 'company')),
+                ('group', '=', Eval('group')),
+            ])
     sequence = fields.Integer('Sequence')
     template = fields.Many2One('account.tax.rule.line.template', 'Template')
 
@@ -1265,6 +1382,12 @@ class RuleLine(ModelSQL, ModelView):
                     rule_line.template, context=context, rule_line=rule_line)
             if rule_line.rule.id != template2rule[rule_line.template.rule.id]:
                 vals['rule'] = template2rule[rule_line.template.rule.id]
+            if rule_line.origin_tax:
+                if rule_line.template.origin_tax:
+                    if rule_line.origin_tax.id != \
+                            template2tax[rule_line.template.origin_tax.id]:
+                        vals['origin_tax'] = template2tax[
+                            rule_line.template.origin_tax.id]
             if rule_line.tax:
                 if rule_line.template.tax:
                     if rule_line.tax.id != \
@@ -1313,23 +1436,21 @@ class OpenCode(Wizard):
         else:
             period_ids = context['periods']
 
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_tax_line_form'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id,
-                context=context)
-        res['domain'] = str([
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_tax_line_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_domain'] = PYSONEncoder().encode([
             ('move_line.period', 'in', period_ids),
             ('code', '=', data['id']),
             ])
         if context.get('fiscalyear'):
-            res['context'] = str({'fiscalyear': context['fiscalyear']})
+            res['pyson_context'] = PYSONEncoder().encode({
+                'fiscalyear': context['fiscalyear'],
+            })
         else:
-            res['context'] = str({'periods': period_ids})
+            res['pyson_context'] = PYSONEncoder().encode({
+                'periods': period_ids,
+            })
         return res
 
 OpenCode()
@@ -1352,7 +1473,7 @@ class AccountTemplate(ModelSQL, ModelView):
     _name = 'account.account.template'
     taxes = fields.Many2Many('account.account.template-account.tax.template',
             'account', 'tax', 'Default Taxes',
-            domain="[('parent', '=', False)]")
+            domain=[('parent', '=', False)])
 
 AccountTemplate()
 
@@ -1374,7 +1495,10 @@ class Account(ModelSQL, ModelView):
     _name = 'account.account'
     taxes = fields.Many2Many('account.account-account.tax',
             'account', 'tax', 'Default Taxes',
-            domain="[('company', '=', company), ('parent', '=', False)]",
+            domain=[
+                ('company', '=', Eval('company')),
+                ('parent', '=', False),
+            ],
             help='Default tax for manual encoding of move lines \n' \
                     'for journal types: "expense" and "revenue"',
             depends=['company'])
